
get_core <- function(path) {
  res <- jsonlite::fromJSON(path, simplifyDataFrame = FALSE)
  # Info element is required
  if(is.null(res$information)) {
    warning("Missing Specification information")
  }
  class(res) <- c("algoli_core", class(res))
  return(res)
}

get_core_functions <- function(core) {

  functions_defs <- get_core_function_definitions(core)

  param_values <- expression({
    if (length(formals()) > 0) {
      l1 <- as.list(mget(names(formals()), environment()))
      l1 <- l1[lapply(l1, mode) != "name"]
      x <- l1[!vapply(l1, is.null, logical(1))]
    } else {
      x <- list()
    }
    x
  })

  lapply(functions_defs, function(fun_def){

    tmp_fun <- function() {
      x <- eval(param_values)
      tmp <- eval(get_function(fun_def))
      do.call("tmp",x)
    }

    parameters <- get_parameters(fun_def$parameters)
    if(length(parameters)) {
      formals(tmp_fun) <- do.call(alist, parameters)
    }

    attr(tmp_fun, "definition") <- fun_def
    class(tmp_fun) <- c("algoli_core_function", class(tmp_fun))
    tmp_fun
  })
}

get_core_function_definitions <- function(core) {
  res <- list()
  action_types <- c("connect", "disconnect","df","copy","split_d","export_model","load_model")
  for(a in intersect(names(core$actions), action_types)) {
    functions <- core$actions[[a]]
    functions$action <- a
    function_id <- paste0(a)
    res <- c(res, stats::setNames(list(functions), function_id))
  }
  return(res)
}
