
get_service <- function(path) {
  res <- jsonlite::fromJSON(path, simplifyDataFrame = FALSE)
  # Info element is required
  if(is.null(res$information)) {
    warning("Missing Specification information")
  }
  class(res) <- c("algoli_service", class(res))
  return(res)
}

get_functions <- function(service) {

  functions_defs <- get_function_definitions(service)

  param_values <- expression({
    if (length(formals()) > 0) {
      l1 <- as.list(mget(names(formals()), environment()))
      l1 <- l1[lapply(l1, mode) != "name"]
      x <- l1[!vapply(l1, is.null, logical(1))]
    } else {
      x <- list()
    }
    x
  })

  lapply(functions_defs, function(fun_def){

    tmp_fun <- function() {
      x <- eval(param_values)
      tmp <- eval(get_function(fun_def))
      do.call("tmp",x)
    }

    parameters <- get_parameters(fun_def$parameters)
    if(length(parameters)) {
      formals(tmp_fun) <- do.call(alist, parameters)
    }

    attr(tmp_fun, "definition") <- fun_def
    class(tmp_fun) <- c("algoli_service_function", class(tmp_fun))
    tmp_fun
  })
}

get_function_definitions <- function(service) {
  res <- list()
  algo_names <- names(service$algorithms)
  service_name <- service$information$name
  for(algo_name in algo_names) {
    action_types <- c("fit", "predict")
    for(action in intersect(names(service$algorithms[[algo_name]]), action_types)) {
      functions <- service$algorithms[[algo_name]][[action]]
      functions$algo <- algo_name
      functions$action <- action
      function_id <- paste0(service$information$name,"_",action,"_",algo_name)
      res <- c(res, stats::setNames(list(functions), function_id))
    }
  }
  return(res)
}

get_function <- function(x){
  parse(text=x$'function')
}

get_parameters <- function(x) {
  parameters <- lapply(x, function(p) {
    par_name <- p$name
  })

  if(length(parameters)) {
    parameters <- unlist(parameters)
    parameters <- stats::setNames(
      vector("list", length(parameters)),
      parameters
    )
  }
  parameters
}
